﻿using System;
using System.Collections.Generic;
using System.Media;
using System.Text;
using TSLab.Script;
using TSLab.Script.Handlers;

namespace RusAlgo.Handlers
{
    [HandlerCategory("RusAlgo")]
    [HandlerName("Beeper")]
    public class Beeper : IOneSourceHandler, IBooleanInputs, IValuesHandlerWithNumber, IBooleanReturns, IContextUses
    {
        private SystemSound _sound;
        private string _soundStr;

        public IContext Context { set; private get; }
        
        [HandlerParameter(Name = "Имя звука", Default = "beep", NotOptimized = true)]
        public string SoundName
        {
            get { return _soundStr; }
            set
            {
                value = value.ToLower();
                switch (value)
                {
                    case "beep":
                        _soundStr = "beep";
                        _sound = SystemSounds.Beep;
                        break;

                    case "asterisk":
                        _soundStr = "asterisk";
                        _sound = SystemSounds.Asterisk;
                        break;

                    case "exclamation":
                        _soundStr = "exclamation";
                        _sound = SystemSounds.Exclamation;
                        break;

                    case "hand":
                        _soundStr = "hand";
                        _sound = SystemSounds.Hand;
                        break;

                    case "question":
                        _soundStr = "question";
                        _sound = SystemSounds.Question;
                        break;

                    default:
                        throw new InvalidOperationException("Не существует такого имени сигнала " + value);
                }
            }
        }

        public bool Execute(bool signal, int barNum)
        {
            if (!signal)
                return signal;

            var lastBarIndex = Context.BarsCount - 1;

            // Для режима пересчета по обычному интервалу, бипать нужно когда сигнал сгенерен на последней свече.
            // Но если мы по тикам, интервал + сделка и так далее, То сигнал не генерится на последней свече, он на предпоследней.
            if (Context.IsLastBarUsed)
            {
                if (barNum == lastBarIndex)
                    _sound.Play();
            }
            else
            {
                if (barNum == lastBarIndex-1)
                    _sound.Play();
            }

            return signal;
        }
    }
}
